@@ -109,7 +109,7 @@ codes <- function(x, layer = "cmp_code") {
 #' @method codes ManifestoDocument
 #' @export
 codes.ManifestoDocument <- function(x, layer = "cmp_code") {
-  return(x$content[,layer])
+  return(unlist(x$content[,layer]))
 }
 
 #' @method codes ManifestoCorpus
