@@ -42,16 +42,9 @@ mp_interpolate <- function(df,
       df %>%
         right_join(data.frame(edate = seq_Date_multi(df$edate, by = by),
                               party = the_party),
-                   by = c("edate", "party")) -> df_2 ## workaround ## %>%
-##        mutate_each_(funs(zoo(., edate) %>% the_approx() %>% as.numeric()),
-##                     vars = grep(vars, names(df), value = TRUE))
-
-      ## workaround for dplyr 0.4.2 bug https://github.com/hadley/dplyr/issues/1228
-      for (varname in grep(vars, names(df_2), value = TRUE)) {
-        df_2[[varname]] <- zoo(df_2[[varname]], df_2[["edate"]]) %>% the_approx() %>% as.numeric()
-      }
-      return(df_2)
-      ## end of workaround
+                   by = c("edate", "party")) %>%
+        mutate_each_(funs(zoo(., edate) %>% the_approx() %>% as.numeric()),
+                     vars = grep(vars, names(df), value = TRUE))
   
     } else {
       return(df)
@@ -219,3 +212,13 @@ median_voter_single <- function(positions,
       as.numeric()
 }
 
+
+#' Compute year from date variable in MPDS
+#' 
+#' @param mpds a dataframe in format of Manifesto Project Main Dataset
+#' @return input data with year variable attached
+#' @export
+attach_year <- function(mpds) {
+  mpds %>%
+    mutate(year = substr(date, 1, 4))
+}
